import os
from datetime import datetime
from typing import TYPE_CHECKING, Literal

if TYPE_CHECKING:
    from nonebot.adapters.onebot.v11 import GroupMessageEvent as GroupMessageEventV11
    from nonebot.adapters.onebot.v11 import (
        PrivateMessageEvent as PrivateMessageEventV11,
    )
    from nonebot.adapters.onebot.v12 import (
        ChannelMessageEvent as ChannelMessageEventV12,
    )
    from nonebot.adapters.onebot.v12 import GroupMessageEvent as GroupMessageEventV12
    from nonebot.adapters.onebot.v12 import (
        PrivateMessageEvent as PrivateMessageEventV12,
    )


def fake_group_message_event_v11(**field) -> "GroupMessageEventV11":
    from nonebot.adapters.onebot.v11 import GroupMessageEvent, Message
    from nonebot.adapters.onebot.v11.event import Sender
    from pydantic import create_model

    _Fake = create_model("_Fake", __base__=GroupMessageEvent)

    class FakeEvent(_Fake):
        time: int = 1000000
        self_id: int = 1
        post_type: Literal["message"] = "message"
        sub_type: str = "normal"
        user_id: int = 10
        message_type: Literal["group"] = "group"
        group_id: int = 10000
        message_id: int = 1
        message: Message = Message("test")
        raw_message: str = "test"
        font: int = 0
        sender: Sender = Sender(
            card="",
            nickname="test",
            role="member",
        )
        to_me: bool = False

        class Config:
            extra = "forbid"

    return FakeEvent(**field)


def fake_private_message_event_v11(**field) -> "PrivateMessageEventV11":
    from nonebot.adapters.onebot.v11 import Message, PrivateMessageEvent
    from nonebot.adapters.onebot.v11.event import Sender
    from pydantic import create_model

    _Fake = create_model("_Fake", __base__=PrivateMessageEvent)

    class FakeEvent(_Fake):
        time: int = 1000000
        self_id: int = 1
        post_type: Literal["message"] = "message"
        sub_type: str = "friend"
        user_id: int = 10
        message_type: Literal["private"] = "private"
        message_id: int = 1
        message: Message = Message("test")
        raw_message: str = "test"
        font: int = 0
        sender: Sender = Sender(nickname="test")
        to_me: bool = False

        class Config:
            extra = "forbid"

    return FakeEvent(**field)


def fake_group_message_event_v12(**field) -> "GroupMessageEventV12":
    from nonebot.adapters.onebot.v12 import GroupMessageEvent, Message
    from nonebot.adapters.onebot.v12.event import BotSelf
    from pydantic import create_model

    _Fake = create_model("_Fake", __base__=GroupMessageEvent)

    class FakeEvent(_Fake):
        self: BotSelf = BotSelf(platform="qq", user_id="test")
        id: str = "1"
        time: datetime = datetime.fromtimestamp(1000000)
        type: Literal["message"] = "message"
        detail_type: Literal["group"] = "group"
        sub_type: str = ""
        message_id: str = "10"
        message: Message = Message("test")
        original_message: Message = Message("test")
        alt_message: str = "test"
        user_id: str = "100"
        group_id: str = "10000"
        to_me: bool = False

        class Config:
            extra = "forbid"

    return FakeEvent(**field)


def fake_private_message_event_v12(**field) -> "PrivateMessageEventV12":
    from nonebot.adapters.onebot.v12 import Message, PrivateMessageEvent
    from nonebot.adapters.onebot.v12.event import BotSelf
    from pydantic import create_model

    _Fake = create_model("_Fake", __base__=PrivateMessageEvent)

    class FakeEvent(_Fake):
        self: BotSelf = BotSelf(platform="qq", user_id="test")
        id: str = "1"
        time: datetime = datetime.fromtimestamp(1000000)
        type: Literal["message"] = "message"
        detail_type: Literal["private"] = "private"
        sub_type: str = ""
        message_id: str = "10"
        message: Message = Message("test")
        original_message: Message = Message("test")
        alt_message: str = "test"
        user_id: str = "100"
        to_me: bool = False

        class Config:
            extra = "forbid"

    return FakeEvent(**field)


def fake_channel_message_event_v12(**field) -> "ChannelMessageEventV12":
    from nonebot.adapters.onebot.v12 import ChannelMessageEvent, Message
    from nonebot.adapters.onebot.v12.event import BotSelf
    from pydantic import create_model

    _Fake = create_model("_Fake", __base__=ChannelMessageEvent)

    class FakeEvent(_Fake):
        self: BotSelf = BotSelf(platform="qq", user_id="test")
        id: str = "1"
        time: datetime = datetime.fromtimestamp(1000000)
        type: Literal["message"] = "message"
        detail_type: Literal["channel"] = "channel"
        sub_type: str = ""
        message_id: str = "10"
        message: Message = Message("test")
        original_message: Message = Message("test")
        alt_message: str = "test"
        user_id: str = "10"
        guild_id: str = "10000"
        channel_id: str = "100000"
        to_me: bool = False

        class Config:
            extra = "forbid"

    return FakeEvent(**field)


def list_files_in_directory(directory, ignored_extensions=None, ignored_files=None):
    """
    获取指定目录下的所有文件名（不包含扩展名），忽略子目录和指定的文件扩展名。

    参数:
        directory (str): 需要列出文件的目录路径。
        ignored_extensions (list): 需要忽略的文件扩展名列表，例如 ['.txt', '.log']。
        ignored_files (list): 需要忽略的文件名列表（不包含扩展名），例如 ['example', 'test']。

    返回:
        list: 包含目录中所有非忽略文件名（不包含扩展名）的列表。
    """
    if ignored_extensions is None:
        ignored_extensions = []
    if ignored_files is None:
        ignored_files = []

    files = []
    for item in os.listdir(directory):
        full_path = os.path.join(directory, item)
        if os.path.isfile(full_path):
            filename, ext = os.path.splitext(item)
            if ext not in ignored_extensions and filename not in ignored_files:
                files.append(filename)
    return files


def list_keys_in_dict(dictionary):
    """
    获取字典中所有的键。

    参数:
        dictionary (dict): 需要列出键的字典。

    返回:
        list: 包含字典中所有键的列表。
    """
    keys = []
    for key in dictionary.keys():
        keys.append(key)
    return keys

